/*************************************\
| MLC AFX by Jacek (4nic8) Malinowski |
+-------------------------------------+
| contact: 4nic8@casiocalc.org        |
\*************************************/

/***************************************
-> Porting to other calcs info:
1) Interpreter works with tokenized code.
code[0] is the first char of the tokenized code. I used a pointer which is connected
with a MLC file in RAM, but you can use normal array. More information about it in token.c

2) Many functions in this file use "uint *i, syntax sx" parameters:
uint *i		- it's a pointer to unsigned int variable with index of current place in
			code (the "array" with tokenized code)
syntax sx	- it's current syntax code (syntax list is in mlc.h) mostly I pass it as 
			a parameter because it will be displayed when some error occur.
			
3) You'll probably have to change only graphics and keyboard functions in the
void do_syntax(uint *i) routine. Read the main MLC readme file to be sure what
MLC commands should do.
****************************************/

/* global variables available in a MLC prog */
function	funct[MAX_FUNCTS];
t_array		array[MAX_ARRAYS];
t_spr		spr[MAX_SPRS];
t_string	str[MAX_STRS];
t_bmp		bmp[MAX_BMPS];
int			ints[MAX_INTS];

int ac_funct; // active function

/* takes positions of functions in MLC code */
void take_functions(uint size){
	uint i;
	syntax sx;	
	for (i=0;i<MAX_FUNCTS;i++){
        funct[i].pos=0;
        funct[i].from=0;
        funct[i].last_funct=0;
    }
	for (i=0;i<size;i++){
		if (code[i]==SYNTAX_TAG)
		{
			sx=code[++i];
			if (sx==0){
				funct[code[++i]].pos=i+2;
			}
		}
	}

#if defined DEBUG3
	clrscr();
	for (i=0;i<5;i++)
		printf("\n%u",funct[i].pos);
	if (getch()==0) getch();	
#endif
		
}

int get_num(uint *i, syntax sx)// gets a number
{
	int sign=1;
	int pom=0;
	int pom2=1;
	int j,l=0;
	uchar num[10];
	if (code[(*i)++]!=NUM_TAG) error_msg_pos((*i),syntax_list[sx],ER_BAD_NUM);
	if (code[(*i)]=='-')
	{
		sign=-1;(*i)++;
	}

	while(code[(*i)] <= 9)
	{ 
		num[pom++]=code[(*i)];
		(*i)++;
	}

	for (j=pom-1;j>=0;j--){
		l+=pom2*num[j];
		pom2*=10;
	}
	l*=sign;
	return l;
}

int get_any_int(uint *i,syntax sx);

#define get_int_inx() if (code[(*i)++]!=INT_TAG) error_msg_pos((*i),syntax_list[sx],ER_BAD_USE)

#define get_array_inx() if (code[(*i)++]!=ARRAY_TAG) error_msg_pos((*i),syntax_list[sx],ER_BAD_USE)

#define get_spr_inx() if (code[(*i)++]!=SPR_TAG) error_msg_pos((*i),syntax_list[sx],ER_BAD_USE)

#define get_str_inx() if (code[(*i)++]!=STR_TAG) error_msg_pos((*i),syntax_list[sx],ER_BAD_USE)

/* gets a value of a cell in an array */
int get_array(uint *i, syntax sx){
	int ix,cell;	
	get_array_inx();
	ix = code[(*i)++]; // gets array's index;
	if (code[(*i)++]!='(') error_msg_pos((*i),ER_BAD_ARRAY,"");
	cell=get_any_int(i,sx);
	if (cell>=array[ix].size || cell<0) error_msg_pos((*i),ER_ARRAY_FAR,"");
	if (code[(*i)++]!=')') error_msg_pos((*i),ER_BAD_ARRAY,"");
    return	array[ix].data[cell];
	

}

int get_spr(uint *i, syntax sx){
	int ix;	
	get_spr_inx();
	ix = code[(*i)++]; // gets spr's index;
    if (code[(*i)]=='.'){
		(*i)++;
		switch (code[(*i)]){
			case 'X':
				(*i)++;
				return spr[ix].x;
			case 'Y':
				(*i)++;
				return spr[ix].y;
			case 'D':
				(*i)++;
				switch (code[(*i)]){
				case 'X':
					(*i)++;
					return spr[ix].dx;
				case 'Y':
					(*i)++;
					return spr[ix].dy;
				}
			break;
			case 'V':
				(*i)++;
				switch (code[(*i)]){
				case 'X':
					(*i)++;
					return spr[ix].vx;
				case 'Y':
					(*i)++;
					return spr[ix].vy;
				}
			break;
			default:
				error_msg_pos((*i),ER_BAD_SPR,"");
		}
	} else error_msg_pos((*i),ER_BAD_SPR,"");	
	return 0;
}

int get_any_int(uint *i,syntax sx){//do math  ex.: %X=.6564+.453/.%Y-.5345
    int num=0;
	char c;
#if defined DEBUG4
	printf("\n%u: %u",(*i),sx);
	if (getch()==0) getch();
#endif
		c=ADD_TAG;
		do{
			switch (code[(*i)])
			{
				case INT_TAG:
					(*i)++;
					switch (c)
					{
						case ADD_TAG:num+=ints[code[(*i)++]];break;
						case SUB_TAG:num-=ints[code[(*i)++]];break;
						case MUL_TAG:num*=ints[code[(*i)++]];break;
						case DIV_TAG:num/=ints[code[(*i)++]];break;
						case RES_TAG:num%=ints[code[(*i)++]];break;
                        default:
                            error_msg_pos((*i),syntax_list[sx],ER_BAD_MATH);
					}
				break;
				case ARRAY_TAG:					
					switch (c)
					{
						case ADD_TAG:num+=get_array(i,sx);break;
						case SUB_TAG:num-=get_array(i,sx);break;
						case MUL_TAG:num*=get_array(i,sx);break;
						case DIV_TAG:num/=get_array(i,sx);break;
						case RES_TAG:num%=get_array(i,sx);break;
                        default:
                            error_msg_pos((*i),syntax_list[sx],ER_BAD_MATH);
					}
				break;
				case NUM_TAG:
					switch (c)
					{
						case ADD_TAG:num+=get_num(i,sx);break;
						case SUB_TAG:num-=get_num(i,sx);break;
						case MUL_TAG:num*=get_num(i,sx);break;
						case DIV_TAG:num/=get_num(i,sx);break;
						case RES_TAG:num%=get_num(i,sx);break;
                        default:
                            error_msg_pos((*i),syntax_list[sx],ER_BAD_MATH);
					}
				break;
				case SPR_TAG:
					switch (c)
					{
						case ADD_TAG:num+=get_spr(i,sx);break;
						case SUB_TAG:num-=get_spr(i,sx);break;
						case MUL_TAG:num*=get_spr(i,sx);break;
						case DIV_TAG:num/=get_spr(i,sx);break;
						case RES_TAG:num%=get_spr(i,sx);break;
                        default:
                            error_msg_pos((*i),syntax_list[sx],ER_BAD_MATH);
					}
				break;
                default:
                    error_msg_pos((*i),syntax_list[sx],ER_BAD_COUNT);
			
			}
			c=code[(*i)++];
		} while (c==ADD_TAG || c==SUB_TAG || c==MUL_TAG || c==DIV_TAG || c==RES_TAG);        
		(*i)--;
        return num;
}


void get_string(uint *i,syntax sx, char *string){//gets a string
	int ix=0;
	if (code[(*i)++]!=STRING_TAG) error_msg_pos((*i),syntax_list[sx],ER_STR_TAG);
		do{
			string[ix++]=code[(*i)++];
			if (ix>=50) error_msg_pos((*i),syntax_list[sx],ER_STR_MAX);
		}while (string[ix-1]!=STRING_TAG);
		string[ix-1]='\0';	
}

void get_str_or_int(uint *i,syntax sx, char *string){
	switch (code[(*i)]){
			case STRING_TAG:
				get_string(i,sx,string);				
			break;
			case INT_TAG:
				get_int_inx();
				sprintf(string,"%d",ints[code[(*i)++]]);								
			break;
			case ARRAY_TAG:
				sprintf(string,"%d",get_array(i,sx));												
			break;
			case SPR_TAG:				
				sprintf(string,"%d",get_spr(i,sx));										
			break;
			case STR_TAG:				
				get_str_inx();
				sprintf(string,"%s",str[code[(*i)++]].str);
			break;
            default:
                error_msg_pos((*i),syntax_list[sx],ER_STR_PAR);
	}

}

void set_int_or_arr(uint *i,syntax sx,int value)
{
	int ix,cell;
	switch (code[(*i)]){
			case INT_TAG:
				get_int_inx();
				ints[code[(*i)++]]=value;				
			break;
			case ARRAY_TAG:
				get_array_inx();
				ix = code[(*i)++]; // gets array's index;
				if (code[(*i)++]!='(') error_msg_pos((*i),ER_BAD_ARRAY,"");
				cell=get_any_int(i,sx);
				if (cell>=array[ix].size || cell<0) error_msg_pos((*i),ER_ARRAY_FAR,"");
				if (code[(*i)++]!=')') error_msg_pos((*i),ER_BAD_ARRAY,"");
			  	array[ix].data[cell]=value;		
			break;
	        default:
		            error_msg_pos((*i),syntax_list[sx],ER_INT_PAR);
	}	
}

int m_if(uint *i,syntax sx){
    int x1;
    int x2;
    char condition;
    x1=get_any_int(i,sx);
    condition=code[(*i)++];
    x2=get_any_int(i,sx);
    switch (condition){
        case EQU_TAG: return (x1==x2);
        case NOTEQU_TAG: return (x1!=x2);
        case SMALLER_TAG: return (x1<x2);
        case BIGGER_TAG: return (x1>x2);
        default:
            error_msg_pos((*i),"Bad condition","");
    }

    return 0;
}

int mlc_if(uint *i,syntax sx){	
	do{
		if (code[(*i)]==AND_TAG) (*i)++;		
		if (!m_if(i,sx)) return 0;		
	} while (code[(*i)]==AND_TAG);

	return 1;
}

void skip_line(uint *i){
	while (code[(*i)++]!=NEXT_L_CODE);
}

void do_syntax(uint *i){

	int x,y,x2,y2,col,col2,tmp,ix,j;
	uint ip,ip2;
	uchar b,b2;
	syntax sx;
	char string[50];
	(*i)++;
#if defined DEBUG4
	printf("\n%u: %u",(*i),code[(*i)]);
	if (getch()==0) getch();
#endif
	if (over) return;
	sx=code[(*i)++];
	switch (sx){
	case WKEY:		
		ip=(*i);
		set_int_or_arr(i,sx,0);
		(*i)=ip;
		while (get_any_int(i,sx)==0){
			(*i)=ip;
			set_int_or_arr(i,sx,tgetkey_fast());
			ip2=(*i);
			(*i)=ip;
			if (over) return;
		}
		(*i)=ip2;

	break;
	case GKEY:
        x=get_any_int(i,sx);
		(*i)++;
		if (testkey(key_row[x],key_col[x])!=0) 
			set_int_or_arr(i,sx,1);
		else
   			set_int_or_arr(i,sx,0);
	break;
    case IF://_IF_ <int> <condition> <int>
        if (!mlc_if(i,sx)){           
           skip_line(i);
		   skip_line(i);
		}
    break;
    case PIXL: // PIXE x y col  sets pixel
		x=get_any_int(i,sx);
		(*i)++;
		y=get_any_int(i,sx);
		(*i)++;
		col=get_any_int(i,sx);		
		put_pix(x,y,col);
	break;
	case FRUN: // FRUN - runs function
		tmp=ac_funct;
		ac_funct=code[(*i)];
		funct[ac_funct].from=(*i)+1;
		funct[ac_funct].last_funct=tmp;
		(*i)=funct[ac_funct].pos;
		#if defined DEBUG4
		printf("\n%u: %u !!!",(*i),code[(*i)]);
		if (getch()==0) getch();
		#endif
	break;
	case FGOB:// FGOB - returns to the begining of function
		(*i)=funct[ac_funct].pos;
	break;
	case FEND:// FEND - end of function
		if (funct[ac_funct].from==999) halt();//halt();       
		else
		{
			(*i)=funct[ac_funct].from;
			ac_funct=funct[ac_funct].last_funct;
		}
	break;	
	case PIXT: // PIXE TEST x y col
		x=get_any_int(i,sx);
		(*i)++;
		y=get_any_int(i,sx);
		(*i)++;
		set_int_or_arr(i,sx,get_pix(x,y));		
	break;	
	case CLRS:// CLRS  clears screen		
        gxClearBuff();
	break;
	case PAUS:
		while(!testkey(K_EXE))
			if (over) return;		
	return;//PAUSE - waits for ENTER/EXE
	case DRAW:		
		gxRefresh();
	break;
	case TEXT:// TEXT x y 'text' col  displays text
		x=get_any_int(i,sx);
		(*i)++;
		y=get_any_int(i,sx);
		(*i)++;
		get_str_or_int(i,sx,string);		
		(*i)++;
		col=get_any_int(i,sx);	
		OutStrTI(x,y,string,col,ti);	
	return;
	case LINE:// LINE x1 y1 x2 y2  displays line
		x=get_any_int(i,sx);
		(*i)++;
		y=get_any_int(i,sx);
		(*i)++;
		x2=get_any_int(i,sx);
		(*i)++;
		y2=get_any_int(i,sx);
		(*i)++;		
		col=get_any_int(i,sx);	
		line(x,y,x2,y2,col);
	return;
	case RECT:// RECT x1 y1 x2 y2  displays rectangle
		x=get_any_int(i,sx);
		(*i)++;
		y=get_any_int(i,sx);
		(*i)++;
		x2=get_any_int(i,sx);
		(*i)++;
		y2=get_any_int(i,sx);
		(*i)++;		
		col=get_any_int(i,sx);
		(*i)++;		
		col2=get_any_int(i,sx);
		if (x2<x) swap(&x,&x2); 
		if (y2<y) swap(&y,&y2); 
		if (col!=5)
			rect(x,y,x2,y2,col);
		if (col2!=5)
			frect(x+1,y+1,x2-1,y2-1,col2);
	break;
	case DBMP:// draws a bitmap   	    
		(*i)++;
		b=code[(*i)];
		(*i)+=2;
		x=get_any_int(i,sx);
		(*i)++;
		y=get_any_int(i,sx);
		if (x>-16 && y>-16 && x<144 && y<80)
			gxSprite(x,y,bmp[b].bmp,GX_SPR_MASK_CLIP);
	break;
	case SPRT://  
		(*i)++;
		b=code[(*i)];
		(*i)+=3;
		spr[b].bmp_index=code[(*i)++];		
	break;
	case DISP:// draws a sprite  	    
		(*i)++;
		b=code[(*i)];
		(*i)+=2;		
		if (spr[b].x>-16 && spr[b].y>-16 && spr[b].x<144 && spr[b].y<80)
			gxSprite(spr[b].x,spr[b].y,bmp[spr[b].bmp_index].bmp,GX_SPR_MASK_CLIP);
		spr[b].dy+=spr[b].vy;
		while (spr[b].dy>=100) {spr[b].dy-=100; spr[b].y++; }
		while (spr[b].dy<=-100) {spr[b].dy+=100; spr[b].y--; }
		spr[b].dx+=spr[b].vx;
		while (spr[b].dx>=100) {spr[b].dx-=100; spr[b].x++; }
		while (spr[b].dx<=-100) {spr[b].dx+=100; spr[b].x--; }
	break;
	case CLSN:// draws a sprite  	    
		(*i)++;
		b=code[(*i)];
		(*i)+=3;	
		b2=code[(*i)];	
		(*i)+=2;
		if (abs(spr[b].x-spr[b2].x)<8 && abs(spr[b].y-spr[b2].y)<8)
			set_int_or_arr(i,sx,1);
		else
			set_int_or_arr(i,sx,0);
	break;
	case FRCT:// RECT x1 y1 x2 y2  displays rectangle
		x=get_any_int(i,sx);
		(*i)++;
		y=get_any_int(i,sx);
		(*i)++;
		x2=get_any_int(i,sx);
		(*i)++;
		y2=get_any_int(i,sx);
		(*i)++;		
		col=get_any_int(i,sx);
		frect(x,y,x2,y2,col);
	break;
	case RNDM:// PIXE x y col  sets pixel
		x=get_any_int(i,sx);
		(*i)++;
		y=get_any_int(i,sx);
		(*i)++;
		set_int_or_arr(i,sx,random(y-x+1)+x);
	break;

	case STOP:		
		backup((uchar far*)scr_b123);
	break;

	case RCLP:		
		restore((uchar far*)scr_b123);	
	break;

	case SHFT:
		x=get_any_int(i,sx);
		(*i)++;
		y=get_any_int(i,sx);
		(*i)++;
		col=get_any_int(i,sx);
		scroll(x,y,col);
	break;
    case HALT:
		halt();
    break;
    case WAIT:
		x=get_any_int(i,sx);
		mlc_wait(x);
    break;
	case WRTE:
		x=get_any_int(i,sx);
		if (x>=MLC_SV_SLOTS || x<0)
			error_msg_pos((*i),syntax_list[sx],ER_IO);
		(*i)++;
		y=get_any_int(i,sx);		
		if (y>0){
			mlc_save[3*x]=0;
			mlc_save[3*x+1]=y/256;
			mlc_save[3*x+2]=y%256;
		}
		else{
			y=-y;
			mlc_save[3*x]=1;
			mlc_save[3*x+1]=y/256;
			mlc_save[3*x+2]=y%256;
		}
    break;
	case READ:
		x=get_any_int(i,sx);
		if (x>=MLC_SV_SLOTS || x<0)
			error_msg_pos((*i),syntax_list[sx],ER_IO);
		(*i)++;
		y=256*mlc_save[3*x+1]+mlc_save[3*x+2];
		if (mlc_save[3*x]>0) y=-y;
		set_int_or_arr(i,sx,y);
    break;

	case ARRY:// ARRAY @NAM size - allocates memory for an array
		get_array_inx();
		ix = code[(*i)++];
		(*i)++;
		y=get_any_int(i,sx);		
		if (array[ix].size!=0) error_msg_pos((*i),syntax_list[sx],ER_ARR_INT);
		array[ix].size=y;		
		if ((array[ix].data=(int *)malloc(y*sizeof(int)))==NULL)
			error_msg_pos((*i),syntax_list[sx],ER_MEM_OUT);
	break;
	case FMEM:
		get_int_inx();
		ints[code[(*i)++]]=(int) coreleft();
    break;
    default:
		error_msg_pos((*i),syntax_list[sx],ER_INTER);
	}
	
}

void set_int(uint *i){ // 
	int ix;
	syntax sx;	
	sx=MAX_SYNTAX;
	get_int_inx();
	ix = code[(*i)++]; // gets int's index;
    if (code[(*i)++]=='='){
		ints[ix]=get_any_int(i,sx);
	} else error_msg_pos((*i),ER_BAD_INT,"");
}

void set_array(uint *i){ 
	int ix,cell;
	syntax sx;	
	sx=MAX_SYNTAX+1;
	get_array_inx();
	ix = code[(*i)++]; // gets array's index;
	if (code[(*i)++]!='(') error_msg_pos((*i),ER_BAD_ARRAY,"");
	cell=get_any_int(i,sx);
	if (cell>=array[ix].size || cell<0) error_msg_pos((*i),ER_ARRAY_FAR,"");
	if (code[(*i)++]!=')') error_msg_pos((*i),ER_BAD_ARRAY,"");
    if (code[(*i)++]=='=')
	{
		array[ix].data[cell]=get_any_int(i,sx);
	} else error_msg_pos((*i),ER_BAD_ARRAY,"");
}

void set_str(uint *i){
	int ix;
	syntax sx;	
	char tmp_d[STR_DL];
	char *tmp;
	char *tmp_s;
	sx=MAX_SYNTAX;
	get_str_inx();
	ix = code[(*i)++]; // gets str's index;
	tmp=tmp_d;
	tmp_s=tmp;
    if (code[(*i)++]=='='){
		do{
			get_str_or_int(i,sx,tmp);		
			while(tmp[0]!='\0') 
				tmp++;
		} while(code[(*i)++]=='+');
		sprintf(str[ix].str,"%s",tmp_s);
	} else error_msg_pos((*i),ER_BAD_STR,"");
}

void set_spr(uint *i){ 
	int ix;
	syntax sx;	
	sx=MAX_SYNTAX+2;
	get_spr_inx();
	ix = code[(*i)++]; // gets spr's index;
    if (code[(*i)]=='.'){
		(*i)++;
		switch (code[(*i)]){
			case 'X':
				(*i)+=2;
				spr[ix].x=get_any_int(i,sx);
			break;
			case 'Y':
				(*i)+=2;
				spr[ix].y=get_any_int(i,sx);
			break;
			case 'D':
				(*i)++;
				switch (code[(*i)]){
					case 'X':
					(*i)+=2;
					spr[ix].dx=get_any_int(i,sx);
				break;
				case 'Y':
					(*i)+=2;
					spr[ix].dy=get_any_int(i,sx);
				break;
				}
			break;
			case 'V':
				(*i)++;
				switch (code[(*i)]){
					case 'X':
					(*i)+=2;
					spr[ix].vx=get_any_int(i,sx);
				break;
				case 'Y':
					(*i)+=2;
					spr[ix].vy=get_any_int(i,sx);
				break;
				}
			break;
			case 'B':
				(*i)++;
				if (code[(*i)++]!='M') error_msg_pos((*i),ER_BAD_SPR,"");
				if (code[(*i)++]!='P') error_msg_pos((*i),ER_BAD_SPR,"");
				(*i)+=2;
				spr[ix].bmp_index=code[(*i)++];				
			break;
		}
	} else error_msg_pos((*i),ER_BAD_SPR,"");
}

void start_inter(){
	uint i;
	struct memory_zone fp;
	char err_s[2];
	key_row[11]=10;key_col[11]=6;
	key_row[12]=10;key_col[12]=5;
	key_row[13]=10;key_col[13]=4;
	key_row[14]=10;key_col[14]=3;
	key_row[15]=10;key_col[15]=2;

	key_row[21]=8;key_col[21]=6;
	key_row[22]=8;key_col[22]=5;
	key_row[23]=8;key_col[23]=4;
	key_row[24]=9;key_col[24]=6;
	key_row[25]=9;key_col[25]=5;
	key_row[26]=9;key_col[26]=3;

	key_row[31]=7;key_col[31]=6;
	key_row[32]=7;key_col[32]=5;
	key_row[33]=7;key_col[33]=4;
	key_row[34]=9;key_col[34]=4;
	
	key_row[41]=6;key_col[41]=6;
	key_row[42]=6;key_col[42]=5;
	key_row[43]=6;key_col[43]=4;
	key_row[44]=6;key_col[44]=3;
	key_row[45]=6;key_col[45]=2;

	key_row[51]=5;key_col[51]=6;
	key_row[52]=5;key_col[52]=5;
	key_row[53]=5;key_col[53]=4;
	key_row[54]=5;key_col[54]=3;
	key_row[55]=5;key_col[55]=2;

	key_row[61]=4;key_col[61]=3;
	key_row[62]=4;key_col[62]=6;
	key_row[63]=4;key_col[63]=5;
	key_row[64]=4;key_col[64]=4;
	key_row[65]=0;key_col[65]=0;

	key_row[71]=3;key_col[71]=3;
	key_row[72]=3;key_col[72]=6;
	key_row[73]=3;key_col[73]=5;
	key_row[74]=3;key_col[74]=4;
	key_row[75]=3;key_col[75]=2;

	key_row[81]=2;key_col[81]=3;
	key_row[82]=2;key_col[82]=6;
	key_row[83]=2;key_col[83]=5;
	key_row[84]=2;key_col[84]=4;
	key_row[85]=2;key_col[85]=2;

	key_row[91]=1;key_col[91]=3;
	key_row[92]=1;key_col[92]=6;
	key_row[93]=1;key_col[93]=5;
	key_row[94]=1;key_col[94]=4;
	key_row[95]=1;key_col[95]=2;


	for (i=0;i<MAX_INTS;i++)
      ints[i]=0;
	/*
	for (i=0;i<MAX_ARRAYS;i++){
      array[i].size=0;
	  free(array[i].data);
	}*/
	i=funct[0].pos; //sets the main function
	funct[0].from=999;
	ac_funct=0;
    randomize();	
	over=0;
	for (;;)
	{
#if defined DEBUG4
	printf("\n%u: %c | %u",i,code[i], code[i]);
	if (getch()==0) getch();
#endif
		switch (code[i]){
			case SYNTAX_TAG:
				do_syntax(&i);
			break;
			case INT_TAG:
				set_int(&i);				
			break;
			case ARRAY_TAG:
				set_array(&i);				
			break;
			case SPR_TAG:
				set_spr(&i);				
			break;
			case STR_TAG:
				set_str(&i);				
			break;
			case NEXT_L_CODE:
				i++;
			break;
			default:
				err_s[0]=code[i];
				err_s[1]='\0';
				error_msg_pos(i,err_s,"Syntax error!");
		}
		if (over){
			/* freeing allocated memory */
			for (i=0;i<MAX_ARRAYS;i++){
				if (array[i].size>0) free(array[i].data);
				array[i].size=0;				
			}
			/*
			for (i=0;i<MAX_BMPS;i++){
				if (bmp[i].bmp!=NULL) free(bmp[i].bmp);
			}
			*/
			/* deleting temporary files */
			if (search_mem_zone(1,TEMP_FILE,&fp)!=-1) 
			{
				create_mem_zone(1,TEMP_FILE,fp.b_size,DELETE_ZONE);
			}
			gxSetMode(false);
			inter(1);
			close_go_menu();
			emptybuf();	
			//exit(0);
			return;
		}
	}
    
}