/*************************************\
| MLC AFX by Jacek (4nic8) Malinowski |
+-------------------------------------+
| contact: 4nic8@casiocalc.org        |
\*************************************/


#if !defined __MLC
#define __MLC
#define MAX_INTS 100
#define MAX_STRS 10
#define MAX_ARRAYS 20
#define MAX_BMPS 50
#define MAX_SPRS 50
#define MAX_FUNCTS 100

#define MAX_NAME 5
#define MAX_F_NAME 5

#define SPR8x8_SIZE 29
#define SPR16x16_SIZE 101

#define TEMP_FILE "~MLCTMP~"
#define TEMP_SCR_FILE "~MLCSCR~"

#define MLC_SAVE_FILE 99+24
#define MLC_SV_SLOTS 33

typedef struct{
	unsigned int pos; // position of function
	unsigned int from; // place from it was run and where it comes back when function is terminated
	unsigned int last_funct; // previous function index (name)
} function;

typedef struct{
	unsigned int size;
	int *data;
} t_array;

typedef struct{
	int size;
	uchar *bmp;//[SPR8x8_SIZE];
	uchar x1,x2,y1,y2;
} t_bmp;

typedef struct{
	int x,y;
	int dx,dy,vx,vy;
	int bmp_index;
} t_spr;

#define STR_DL 50

typedef struct{
	int length;
	char str[STR_DL];
} t_string;

#define SYNTAX_TAG	'#'
#define INT_TAG		'%'
#define NUM_TAG		'.'
#define ARRAY_TAG	'@'
#define STR_TAG		'$'
#define BMP_TAG		'['
#define SPR_TAG		'^'

#define ADD_TAG		'+'
#define SUB_TAG		'-'
#define MUL_TAG		'*'
#define DIV_TAG		'/'
#define RES_TAG		'|'

#define EQU_TAG		'='
#define NOTEQU_TAG	'!'
#define SMALLER_TAG '<'
#define BIGGER_TAG	'>'
#define AND_TAG		'&'


#define STRING_TAG	'"'


#define MAX_OP 13

char op_list[]={RES_TAG,NUM_TAG,ADD_TAG,SUB_TAG,MUL_TAG,DIV_TAG,EQU_TAG,
				NOTEQU_TAG,SMALLER_TAG,BIGGER_TAG,AND_TAG,')','('};

#define FUNCT_TAG "FNCT"
#define FUNCT_END_TAG "FEND"
#define RUN_FUNCT_TAG "FRUN"
#define HALT_TAG "HALT"
#define GOTO_B_TAG "FGOB"
#define PAUSE_TAG "PAUS"
#define TEXT_TAG "TEXT"
#define LINE_TAG "LINE"
#define RECT_TAG "RECT"
#define PIXEL_TAG "PIXL"
#define PIXEL_T_TAG "PIXT"
#define CLRSCR_TAG "CLRS"
#define WAITKEY_TAG "WKEY"
#define GETKEY_TAG "GKEY"
#define IF_TAG "IIFF"
#define RAND_TAG "RNDM"
#define WAIT_TAG "WAIT"
#define DRAW_TAG "DRAW"
#define FILL_RECT_TAG "FRCT"
#define ARRAY_DEF_TAG "ARRY"
#define FMEM_TAG "FMEM"
#define STOP_TAG "STOP"
#define RCLP_TAG "RCLP"
#define SHFT_TAG "SHFT"

#define DATA_TAG "DATA"
#define DEND_TAG "DEND"
#define BITM_TAG "BITM"

#define DBMP_TAG "DBMP"

#define DISP_TAG "DISP"
#define SPRT_TAG "SPRT"
#define CLSN_TAG "CLSN"

#define WRITE_TAG "WRTE"
#define READ_TAG "READ"


#define MAX_SYNTAX 33

char syntax_list[][5]={
	FUNCT_TAG,FUNCT_END_TAG,
	RUN_FUNCT_TAG,HALT_TAG,
	GOTO_B_TAG,PAUSE_TAG,
	TEXT_TAG,LINE_TAG,
	RECT_TAG,PIXEL_TAG,
	PIXEL_T_TAG,CLRSCR_TAG,
	WAITKEY_TAG,GETKEY_TAG,
	IF_TAG,RAND_TAG,
	WAIT_TAG,DRAW_TAG,
	FILL_RECT_TAG,ARRAY_DEF_TAG,
	FMEM_TAG,STOP_TAG,
	RCLP_TAG,SHFT_TAG,
	DATA_TAG,DEND_TAG,
	BITM_TAG,DBMP_TAG,
	DISP_TAG,SPRT_TAG,
	CLSN_TAG,WRITE_TAG,
	READ_TAG,
	"\%INT","@TAB","^SPR"
};

typedef enum {FNCT=0,FEND=1,FRUN=2,HALT=3,FGOB=4,PAUS=5,TEXT=6,
			  LINE=7,RECT=8,PIXL=9,PIXT=10,CLRS=11, WKEY=12, GKEY=13,
			  IF=14,RNDM=15,WAIT=16,DRAW=17,FRCT=18, ARRY=19, FMEM=20,
			  STOP=21,RCLP=22, SHFT=23, DATA=24, DEND=25, BITM=26, DBMP=27,
			  DISP=28,SPRT=29, CLSN=30, WRTE=31, READ=32 } syntax;


uchar key_row[95];
uchar key_col[95];

#define STRING_TAG '"'

#define NEXT_L_TAG 13
#define NEXT_L_CODE 255

/* ERRORS: */

#define ER_REDEC		"Redeclaration of function's name. Type different name instead." 
#define ER_BADNAME_F	"Bad function name! Up to 5 letters only." 
#define ER_BADNAME_I    "Bad int name! Up to 5 letters only." 
#define ER_BADNAME_ST    "Bad srting name! Up to 5 letters only." 
#define ER_BADNAME_S    "Bad sprite name! Up to 5 letters only." 
#define ER_BADNAME_A    "Bad array name! Up to 5 letters only." 
#define ER_TOO_INT      "The maximum number of integer variables is 255." 
#define ER_TOO_BMP      "The maximum number of BMPs is 50."
#define ER_TOO_SPR      "The maximum number of sprites is 50."
#define ER_TOO_STR      "The maximum number of strings is 10."
#define ER_TOO_FNCT		"The maximum number of functions is 100." 
#define ER_TOO_ARRAY	"The maximum number of arrays is 20."
  
#define ER_SYNTAX		"Syntax error! Unknown command." 
#define ER_COMPIL		"Error during compiling! It may be some extra space." 
#define ER_BAD_USE		"Bad use of command." 
#define ER_BAD_NUM		"Bad number format. You forgot about '.'" 
#define ER_BAD_MATH     "Bad math operation. You can use only +,-,*,/" 
#define ER_BAD_COUNT	"Error in counting operation. It may be an exta space or word." 
#define ER_NO_FNCT		"FRUN is trying to execute non-existing function" 
#define ER_NO_BMP		"DBMP is trying to draw non-existing bitmap" 
#define ER_STR_TAG		"Missing quotation mark (\")" 
#define ER_STR_MAX		"Sring length too long. Probably you have forgot about ." 
#define ER_STR_PAR		"Bad parameter! It should be int or string." 
#define ER_INT_PAR		"Bad parameter! It should be int or array value."
#define ER_INTER		"Interpreter syntax error!" 
#define ER_BAD_INT		"Bad use of an integer."
#define ER_BAD_SPR		"Bad use of a sprite."
#define ER_BAD_STR		"Bad use of a string."
#define ER_BAD_ARRAY	"Bad use of an array."
#define ER_NO_MAIN		"There is no MAIN function!"
#define ER_MEM_OUT		"Out of memory! Try to make smaller array."
#define ER_ARRAY_FAR	"array subscript out of bounds or attempted access to an un-initialized array."
#define ER_ARR_INT		"array already initialized, cannot re-initialize an existing array."
#define ER_DATA			"bad DATA section!"
#define ER_DEND			"out of file! Probably no #DEND"
#define ER_BMP			"bad bmp format!"
#define ER_BMP_NAME		"bmp name has to start with '[' char."
#define ER_IO			"Bad index of a slot in a mlc save file."

#define TEST_FILE "TEST"

#endif

